@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_variables.c 302150 2010-08-12 17:27:16Z sas $ */
+/* $Id: zend_variables.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include <stdio.h>
 #include "zend.h"
@@ -26,9 +26,10 @@
 #include "zend_constants.h"
 #include "zend_list.h"
 
-ZEND_API void _zval_dtor_func(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
+
+ZEND_API void _zval_dtor_func(zval *zvalue ZEND_FILE_LINE_DC)
 {
-	switch (zvalue->type & ~IS_CONSTANT_INDEX) {
+	switch (Z_TYPE_P(zvalue) & IS_CONSTANT_TYPE_MASK) {
 		case IS_STRING:
 		case IS_CONSTANT:
 			CHECK_ZVAL_STRING_REL(zvalue);
@@ -68,15 +69,15 @@ ZEND_API void _zval_dtor_func(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
 			break;
 	}
 }
-/* }}} */
 
-ZEND_API void _zval_internal_dtor(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
+
+ZEND_API void _zval_internal_dtor(zval *zvalue ZEND_FILE_LINE_DC)
 {
-	switch (zvalue->type & ~IS_CONSTANT_INDEX) {
+	switch (Z_TYPE_P(zvalue) & IS_CONSTANT_TYPE_MASK) {
 		case IS_STRING:
 		case IS_CONSTANT:
 			CHECK_ZVAL_STRING_REL(zvalue);
-			free(zvalue->value.str.val);
+			str_free(zvalue->value.str.val);
 			break;
 		case IS_ARRAY:
 		case IS_CONSTANT_ARRAY:
@@ -92,17 +93,17 @@ ZEND_API void _zval_internal_dtor(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
 			break;
 	}
 }
-/* }}} */
 
-ZEND_API void zval_add_ref(zval **p) /* {{{ */
+
+ZEND_API void zval_add_ref(zval **p)
 {
-	(*p)->refcount++;
+	Z_ADDREF_PP(p);
 }
-/* }}} */
 
-ZEND_API void _zval_copy_ctor_func(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
+
+ZEND_API void _zval_copy_ctor_func(zval *zvalue ZEND_FILE_LINE_DC)
 {
-	switch (zvalue->type) {
+	switch (Z_TYPE_P(zvalue) & IS_CONSTANT_TYPE_MASK) {
 		case IS_RESOURCE: {
 				TSRMLS_FETCH();
 
@@ -116,7 +117,9 @@ ZEND_API void _zval_copy_ctor_func(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
 		case IS_CONSTANT:
 		case IS_STRING:
 			CHECK_ZVAL_STRING_REL(zvalue);
-			zvalue->value.str.val = (char *) estrndup_rel(zvalue->value.str.val, zvalue->value.str.len);
+			if (!IS_INTERNED(zvalue->value.str.val)) {
+				zvalue->value.str.val = (char *) estrndup_rel(zvalue->value.str.val, zvalue->value.str.len);
+			}
 			break;
 		case IS_ARRAY:
 		case IS_CONSTANT_ARRAY: {
@@ -142,58 +145,93 @@ ZEND_API void _zval_copy_ctor_func(zval *zvalue ZEND_FILE_LINE_DC) /* {{{ */
 			break;
 	}
 }
-/* }}} */
 
-ZEND_API int zend_print_variable(zval *var) /* {{{ */
+
+ZEND_API int zend_print_variable(zval *var) 
 {
 	return zend_print_zval(var, 0);
 }
-/* }}} */
 
-ZEND_API void zval_property_ctor(zval **p) /* {{{ */
+
+ZEND_API void _zval_dtor_wrapper(zval *zvalue)
 {
-	zval *orig_ptr = *p;
+	TSRMLS_FETCH();
 
-	ALLOC_ZVAL(*p);
-	**p = *orig_ptr;
-	zval_copy_ctor(*p);
-	(*p)->refcount = 1;
-	(*p)->is_ref   = 0;
+	GC_REMOVE_ZVAL_FROM_BUFFER(zvalue);
+	zval_dtor(zvalue);
 }
-/* }}} */
+
 
 #if ZEND_DEBUG
-ZEND_API void _zval_copy_ctor_wrapper(zval *zvalue) /* {{{ */
+ZEND_API void _zval_copy_ctor_wrapper(zval *zvalue)
 {
 	zval_copy_ctor(zvalue);
 }
-/* }}} */
 
-ZEND_API void _zval_dtor_wrapper(zval *zvalue) /* {{{ */
-{
-	zval_dtor(zvalue);
-}
-/* }}} */
 
-ZEND_API void _zval_internal_dtor_wrapper(zval *zvalue) /* {{{ */
+ZEND_API void _zval_internal_dtor_wrapper(zval *zvalue)
 {
 	zval_internal_dtor(zvalue);
 }
-/* }}} */
 
-ZEND_API void _zval_ptr_dtor_wrapper(zval **zval_ptr) /* {{{ */
+
+ZEND_API void _zval_ptr_dtor_wrapper(zval **zval_ptr)
 {
 	zval_ptr_dtor(zval_ptr);
 }
-/* }}} */
 
-ZEND_API void _zval_internal_ptr_dtor_wrapper(zval **zval_ptr) /* {{{ */
+
+ZEND_API void _zval_internal_ptr_dtor_wrapper(zval **zval_ptr)
 {
 	zval_internal_ptr_dtor(zval_ptr);
 }
-/* }}} */
 #endif
 
+ZEND_API int zval_copy_static_var(zval **p TSRMLS_DC, int num_args, va_list args, zend_hash_key *key) /* {{{ */
+{
+	HashTable *target = va_arg(args, HashTable*);
+	zend_bool is_ref;
+	zval *tmp;
+  
+	if (Z_TYPE_PP(p) & (IS_LEXICAL_VAR|IS_LEXICAL_REF)) {
+		is_ref = Z_TYPE_PP(p) & IS_LEXICAL_REF;
+    
+		if (!EG(active_symbol_table)) {
+			zend_rebuild_symbol_table(TSRMLS_C);
+		}
+		if (zend_hash_quick_find(EG(active_symbol_table), key->arKey, key->nKeyLength, key->h, (void **) &p) == FAILURE) {
+			if (is_ref) {        
+				ALLOC_INIT_ZVAL(tmp);
+				Z_SET_ISREF_P(tmp);
+				zend_hash_quick_add(EG(active_symbol_table), key->arKey, key->nKeyLength, key->h, &tmp, sizeof(zval*), (void**)&p);
+			} else {
+				tmp = EG(uninitialized_zval_ptr);
+				zend_error(E_NOTICE,"Undefined variable: %s", key->arKey);
+			}
+		} else {
+			if (is_ref) {
+				SEPARATE_ZVAL_TO_MAKE_IS_REF(p);
+				tmp = *p;
+			} else if (Z_ISREF_PP(p)) {
+				ALLOC_INIT_ZVAL(tmp);
+				ZVAL_COPY_VALUE(tmp, *p);
+				zval_copy_ctor(tmp);
+				Z_SET_REFCOUNT_P(tmp, 0);
+				Z_UNSET_ISREF_P(tmp);
+			} else {
+				tmp = *p;
+			}
+		}
+	} else {
+		tmp = *p;
+	}
+	if (zend_hash_quick_add(target, key->arKey, key->nKeyLength, key->h, &tmp, sizeof(zval*), NULL) == SUCCESS) {
+		Z_ADDREF_P(tmp);
+	}
+	return ZEND_HASH_APPLY_KEEP;
+}
+/* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
