@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    |          Wez Furlong               <wez@thebrainroom.com>            |
    +----------------------------------------------------------------------+
  */
-/* $Id: calendar.c,v 1.46.2.2.2.2 2007/01/01 09:35:48 sebastian Exp $ */
+/* $Id: calendar.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,29 +36,24 @@
 #include <stdio.h>
 
 /* {{{ arginfo */
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_unixtojd, 0, 0, 0)
 	ZEND_ARG_INFO(0, timestamp)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jdtounix, 0)
 	ZEND_ARG_INFO(0, jday)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_cal_info, 0, 0, 0)
 	ZEND_ARG_INFO(0, calendar)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_cal_days_in_month, 0)
 	ZEND_ARG_INFO(0, calendar)
 	ZEND_ARG_INFO(0, month)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_cal_to_jd, 0)
 	ZEND_ARG_INFO(0, calendar)
 	ZEND_ARG_INFO(0, month)
@@ -66,80 +61,67 @@ ZEND_BEGIN_ARG_INFO(arginfo_cal_to_jd, 0)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_cal_from_jd, 0)
 	ZEND_ARG_INFO(0, jd)
 	ZEND_ARG_INFO(0, calendar)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jdtogregorian, 0)
 	ZEND_ARG_INFO(0, juliandaycount)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_gregoriantojd, 0)
 	ZEND_ARG_INFO(0, month)
 	ZEND_ARG_INFO(0, day)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jdtojulian, 0)
 	ZEND_ARG_INFO(0, juliandaycount)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_juliantojd, 0)
 	ZEND_ARG_INFO(0, month)
 	ZEND_ARG_INFO(0, day)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_jdtojewish, 0, 0, 1)
 	ZEND_ARG_INFO(0, juliandaycount)
 	ZEND_ARG_INFO(0, hebrew)
 	ZEND_ARG_INFO(0, fl)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jewishtojd, 0)
 	ZEND_ARG_INFO(0, month)
 	ZEND_ARG_INFO(0, day)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jdtofrench, 0)
 	ZEND_ARG_INFO(0, juliandaycount)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_frenchtojd, 0)
 	ZEND_ARG_INFO(0, month)
 	ZEND_ARG_INFO(0, day)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_jddayofweek, 0, 0, 1)
 	ZEND_ARG_INFO(0, juliandaycount)
 	ZEND_ARG_INFO(0, mode)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_jdmonthname, 0)
 	ZEND_ARG_INFO(0, juliandaycount)
 	ZEND_ARG_INFO(0, mode)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_easter_date, 0, 0, 0)
 	ZEND_ARG_INFO(0, year)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO_EX(arginfo_easter_days, 0, 0, 0)
 	ZEND_ARG_INFO(0, year)
 	ZEND_ARG_INFO(0, method)
@@ -147,7 +129,7 @@ ZEND_END_ARG_INFO()
 
 /* }}} */
 
-zend_function_entry calendar_functions[] = {
+const zend_function_entry calendar_functions[] = {
 	PHP_FE(jdtogregorian, arginfo_jdtogregorian)
 	PHP_FE(gregoriantojd, arginfo_gregoriantojd)
 	PHP_FE(jdtojulian, arginfo_jdtojulian)
@@ -166,7 +148,7 @@ zend_function_entry calendar_functions[] = {
 	PHP_FE(cal_from_jd, arginfo_cal_from_jd)
 	PHP_FE(cal_days_in_month, arginfo_cal_days_in_month)
 	PHP_FE(cal_info, arginfo_cal_info)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 
@@ -366,8 +348,15 @@ PHP_FUNCTION(cal_days_in_month)
 	sdn_next = calendar->to_jd(year, 1 + month, 1);
 
 	if (sdn_next == 0) {
-/* if invalid, try first month of the next year... */
-		sdn_next = calendar->to_jd(year + 1, 1, 1);
+		/* If the next month is invalid, then we need to try the first month of
+		 * the next year, bearing in mind that the next year after 1 BCE is
+		 * actually 1 AD and not 0. */
+		if (year == -1) {
+			sdn_next = calendar->to_jd(1, 1, 1);
+		}
+		else {
+			sdn_next = calendar->to_jd(year + 1, 1, 1);
+		}
 	}
 
 	RETURN_LONG(sdn_next - sdn_start);
@@ -416,7 +405,7 @@ PHP_FUNCTION(cal_from_jd)
 
 	calendar->from_jd(jd, &year, &month, &day);
 
-	sprintf(date, "%i/%i/%i", month, day, year);
+	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
 	add_assoc_string(return_value, "date", date, 1);
 
 	add_assoc_long(return_value, "month", month);
@@ -447,7 +436,7 @@ PHP_FUNCTION(jdtogregorian)
 	}
 
 	SdnToGregorian(julday, &year, &month, &day);
-	sprintf(date, "%i/%i/%i", month, day, year);
+	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
@@ -480,7 +469,7 @@ PHP_FUNCTION(jdtojulian)
 	}
 
 	SdnToJulian(julday, &year, &month, &day);
-	sprintf(date, "%i/%i/%i", month, day, year);
+	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
@@ -602,7 +591,7 @@ PHP_FUNCTION(jdtojewish)
 	long julday, fl = 0;
 	zend_bool heb   = 0;
 	int year, month, day;
-	char date[16], hebdate[25];
+	char date[16], hebdate[32];
 	char *dayp, *yearp;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|bl", &julday, &heb, &fl) == FAILURE) {
@@ -611,7 +600,7 @@ PHP_FUNCTION(jdtojewish)
 
 	SdnToJewish(julday, &year, &month, &day);
 	if (!heb) {
-		sprintf(date, "%i/%i/%i", month, day, year);
+		snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
 		RETURN_STRING(date, 1);
 	} else {
 		if (year <= 0 || year > 9999) {
@@ -619,7 +608,7 @@ PHP_FUNCTION(jdtojewish)
 			RETURN_FALSE;
 		}
 
-		sprintf(hebdate, "%s %s %s", heb_number_to_chars(day, fl, &dayp), JewishMonthHebName[month], heb_number_to_chars(year, fl, &yearp));
+		snprintf(hebdate, sizeof(hebdate), "%s %s %s", heb_number_to_chars(day, fl, &dayp), JewishMonthHebName[month], heb_number_to_chars(year, fl, &yearp));
 
 		if (dayp) {
 			efree(dayp);
@@ -661,7 +650,7 @@ PHP_FUNCTION(jdtofrench)
 	}
 
 	SdnToFrench(julday, &year, &month, &day);
-	sprintf(date, "%i/%i/%i", month, day, year);
+	snprintf(date, sizeof(date), "%i/%i/%i", month, day, year);
 
 	RETURN_STRING(date, 1);
 }
