@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: pspell.c,v 1.45.2.4.2.5 2007/01/16 11:19:10 tony2001 Exp $ */
+/* $Id: pspell.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #define IS_EXT_MODULE
 
@@ -72,29 +72,133 @@ static PHP_FUNCTION(pspell_config_data_dir);
 static PHP_FUNCTION(pspell_config_repl);
 static PHP_FUNCTION(pspell_config_save_repl);
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_new, 0, 0, 1)
+	ZEND_ARG_INFO(0, language)
+	ZEND_ARG_INFO(0, spelling)
+	ZEND_ARG_INFO(0, jargon)
+	ZEND_ARG_INFO(0, encoding)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_new_personal, 0, 0, 2)
+	ZEND_ARG_INFO(0, personal)
+	ZEND_ARG_INFO(0, language)
+	ZEND_ARG_INFO(0, spelling)
+	ZEND_ARG_INFO(0, jargon)
+	ZEND_ARG_INFO(0, encoding)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_new_config, 0, 0, 1)
+	ZEND_ARG_INFO(0, config)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_check, 0, 0, 2)
+	ZEND_ARG_INFO(0, pspell)
+	ZEND_ARG_INFO(0, word)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_suggest, 0, 0, 2)
+	ZEND_ARG_INFO(0, pspell)
+	ZEND_ARG_INFO(0, word)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_store_replacement, 0, 0, 3)
+	ZEND_ARG_INFO(0, pspell)
+	ZEND_ARG_INFO(0, misspell)
+	ZEND_ARG_INFO(0, correct)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_add_to_personal, 0, 0, 2)
+	ZEND_ARG_INFO(0, pspell)
+	ZEND_ARG_INFO(0, word)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_add_to_session, 0, 0, 2)
+	ZEND_ARG_INFO(0, pspell)
+	ZEND_ARG_INFO(0, word)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_clear_session, 0, 0, 1)
+	ZEND_ARG_INFO(0, pspell)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_save_wordlist, 0, 0, 1)
+	ZEND_ARG_INFO(0, pspell)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_create, 0, 0, 1)
+	ZEND_ARG_INFO(0, language)
+	ZEND_ARG_INFO(0, spelling)
+	ZEND_ARG_INFO(0, jargon)
+	ZEND_ARG_INFO(0, encoding)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_runtogether, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, runtogether)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_mode, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, mode)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_ignore, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, ignore)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_personal, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, personal)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_dict_dir, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, directory)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_data_dir, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, directory)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_repl, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, repl)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_pspell_config_save_repl, 0, 0, 2)
+	ZEND_ARG_INFO(0, conf)
+	ZEND_ARG_INFO(0, save)
+ZEND_END_ARG_INFO()
+/* }}} */
+
 /* {{{ pspell_functions[]
  */
-static zend_function_entry pspell_functions[] = {
-	PHP_FE(pspell_new,		NULL)
-	PHP_FE(pspell_new_personal,		NULL)
-	PHP_FE(pspell_new_config,		NULL)
-	PHP_FE(pspell_check,		NULL)
-	PHP_FE(pspell_suggest,		NULL)
-	PHP_FE(pspell_store_replacement,		NULL)
-	PHP_FE(pspell_add_to_personal,		NULL)
-	PHP_FE(pspell_add_to_session,		NULL)
-	PHP_FE(pspell_clear_session,		NULL)
-	PHP_FE(pspell_save_wordlist,		NULL)
-	PHP_FE(pspell_config_create,		NULL)
-	PHP_FE(pspell_config_runtogether,		NULL)
-	PHP_FE(pspell_config_mode,		NULL)
-	PHP_FE(pspell_config_ignore,		NULL)
-	PHP_FE(pspell_config_personal,		NULL)
-	PHP_FE(pspell_config_dict_dir,		NULL)
-	PHP_FE(pspell_config_data_dir,		NULL)
-	PHP_FE(pspell_config_repl,		NULL)
-	PHP_FE(pspell_config_save_repl,		NULL)
-	{NULL, NULL, NULL} 
+static const zend_function_entry pspell_functions[] = {
+	PHP_FE(pspell_new,					arginfo_pspell_new)
+	PHP_FE(pspell_new_personal,			arginfo_pspell_new_personal)
+	PHP_FE(pspell_new_config,			arginfo_pspell_new_config)
+	PHP_FE(pspell_check,				arginfo_pspell_check)
+	PHP_FE(pspell_suggest,				arginfo_pspell_suggest)
+	PHP_FE(pspell_store_replacement,	arginfo_pspell_store_replacement)
+	PHP_FE(pspell_add_to_personal,		arginfo_pspell_add_to_personal)
+	PHP_FE(pspell_add_to_session,		arginfo_pspell_add_to_session)
+	PHP_FE(pspell_clear_session,		arginfo_pspell_clear_session)
+	PHP_FE(pspell_save_wordlist,		arginfo_pspell_save_wordlist)
+	PHP_FE(pspell_config_create,		arginfo_pspell_config_create)
+	PHP_FE(pspell_config_runtogether,	arginfo_pspell_config_runtogether)
+	PHP_FE(pspell_config_mode,			arginfo_pspell_config_mode)
+	PHP_FE(pspell_config_ignore,		arginfo_pspell_config_ignore)
+	PHP_FE(pspell_config_personal,		arginfo_pspell_config_personal)
+	PHP_FE(pspell_config_dict_dir,		arginfo_pspell_config_dict_dir)
+	PHP_FE(pspell_config_data_dir,		arginfo_pspell_config_data_dir)
+	PHP_FE(pspell_config_repl,			arginfo_pspell_config_repl)
+	PHP_FE(pspell_config_save_repl,		arginfo_pspell_config_save_repl)
+	PHP_FE_END
 };
 /* }}} */
 
@@ -124,18 +228,16 @@ static void php_pspell_close_config(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 }
 
 #define PSPELL_FETCH_CONFIG \
-	convert_to_long_ex(conf);	\
-	config = (PspellConfig *) zend_list_find(Z_LVAL_PP(conf), &type);	\
+	config = (PspellConfig *) zend_list_find(conf, &type);	\
 	if (config == NULL || type != le_pspell_config) {	\
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%ld is not a PSPELL config index", Z_LVAL_PP(conf));	\
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%ld is not a PSPELL config index", conf);	\
 		RETURN_FALSE;	\
 	}	\
 
 #define PSPELL_FETCH_MANAGER \
-	convert_to_long_ex(scin);	\
-	manager = (PspellManager *) zend_list_find(Z_LVAL_PP(scin), &type);	\
+	manager = (PspellManager *) zend_list_find(scin, &type);	\
 	if (!manager || type != le_pspell) {	\
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%ld is not a PSPELL result index", Z_LVAL_PP(scin));	\
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%ld is not a PSPELL result index", scin);	\
 		RETURN_FALSE;	\
 	}	\
 
@@ -157,9 +259,10 @@ static PHP_MINIT_FUNCTION(pspell)
    Load a dictionary */
 static PHP_FUNCTION(pspell_new)
 {
-	zval **language,**spelling,**jargon,**encoding,**pmode;
+	char *language, *spelling = NULL, *jargon = NULL, *encoding = NULL;
+	int language_len, spelling_len = 0, jargon_len = 0, encoding_len = 0;
 	long mode = 0L,  speed = 0L;
-	int argc;
+	int argc = ZEND_NUM_ARGS();
 	int ind;
 
 #ifdef PHP_WIN32
@@ -174,9 +277,9 @@ static PHP_FUNCTION(pspell_new)
 	PspellManager *manager;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc < 1 || argc > 5 || zend_get_parameters_ex(argc,&language,&spelling,&jargon,&encoding,&pmode) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(argc TSRMLS_CC, "s|sssl", &language, &language_len, &spelling, &spelling_len,
+		&jargon, &jargon_len, &encoding, &encoding_len, &mode) == FAILURE) {
+		return;
 	}
 
 	config = new_pspell_config();
@@ -185,60 +288,51 @@ static PHP_FUNCTION(pspell_new)
 	/* If aspell was installed using installer, we should have a key
 	 * pointing to the location of the dictionaries
 	 */
-	if(0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+	if (0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+		LONG result;
 		dwLen = sizeof(aspell_dir) - 1;
-		RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
+		result = RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
 		RegCloseKey(hkey);
-		strcpy(data_dir, aspell_dir);
-		strcat(data_dir, "\\data");
-		strcpy(dict_dir, aspell_dir);
-		strcat(dict_dir, "\\dict");
+		if (result == ERROR_SUCCESS) {
+			strlcpy(data_dir, aspell_dir, sizeof(data_dir));
+			strlcat(data_dir, "\\data", sizeof(data_dir));
+			strlcpy(dict_dir, aspell_dir, sizeof(dict_dir));
+			strlcat(dict_dir, "\\dict", sizeof(dict_dir));
 
-		pspell_config_replace(config, "data-dir", data_dir);
-		pspell_config_replace(config, "dict-dir", dict_dir);
+			pspell_config_replace(config, "data-dir", data_dir);
+			pspell_config_replace(config, "dict-dir", dict_dir);
+		}
 	}
 #endif
 
-	convert_to_string_ex(language);
-	pspell_config_replace(config, "language-tag", Z_STRVAL_PP(language));
+	pspell_config_replace(config, "language-tag", language);
 
-	if(argc > 1){
-		convert_to_string_ex(spelling);
-	 	if(Z_STRLEN_PP(spelling) > 0){
-			pspell_config_replace(config, "spelling", Z_STRVAL_PP(spelling));
-		}
+	if (spelling_len) {
+		pspell_config_replace(config, "spelling", spelling);
 	}
 
-	if(argc > 2){
-		convert_to_string_ex(jargon);
-		if(Z_STRLEN_PP(jargon) > 0){
-			pspell_config_replace(config, "jargon", Z_STRVAL_PP(jargon));
-		}
+	if (jargon_len) {
+		pspell_config_replace(config, "jargon", jargon);
 	}
 
-	if(argc > 3){
-		convert_to_string_ex(encoding);
-		if(Z_STRLEN_PP(encoding) > 0){
-			pspell_config_replace(config, "encoding", Z_STRVAL_PP(encoding));
-		}
+	if (encoding_len) {
+		pspell_config_replace(config, "encoding", encoding);
 	}
 
-	if(argc > 4){
-		convert_to_long_ex(pmode);
-		mode = Z_LVAL_PP(pmode);
+	if (argc > 4) {
 		speed = mode & PSPELL_SPEED_MASK_INTERNAL;
 
 		/* First check what mode we want (how many suggestions) */
-		if(speed == PSPELL_FAST){
+		if (speed == PSPELL_FAST) {
 			pspell_config_replace(config, "sug-mode", "fast");
-		}else if(speed == PSPELL_NORMAL){
+		} else if (speed == PSPELL_NORMAL) {
 			pspell_config_replace(config, "sug-mode", "normal");
-		}else if(speed == PSPELL_BAD_SPELLERS){
+		} else if (speed == PSPELL_BAD_SPELLERS) {
 			pspell_config_replace(config, "sug-mode", "bad-spellers");
 		}
 		
 		/* Then we see if run-together words should be treated as valid components */
-		if(mode & PSPELL_RUN_TOGETHER){
+		if (mode & PSPELL_RUN_TOGETHER) {
 			pspell_config_replace(config, "run-together", "true");
 		}
 	}
@@ -246,14 +340,14 @@ static PHP_FUNCTION(pspell_new)
 	ret = new_pspell_manager(config);
 	delete_pspell_config(config);
 
-	if(pspell_error_number(ret) != 0){
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+	if (pspell_error_number(ret) != 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s", pspell_error_message(ret));
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
 	manager = to_pspell_manager(ret);
-	ind = zend_list_insert(manager, le_pspell);
+	ind = zend_list_insert(manager, le_pspell TSRMLS_CC);
 	RETURN_LONG(ind);
 }
 /* }}} */
@@ -262,9 +356,10 @@ static PHP_FUNCTION(pspell_new)
    Load a dictionary with a personal wordlist*/
 static PHP_FUNCTION(pspell_new_personal)
 {
-	zval **personal, **language,**spelling,**jargon,**encoding,**pmode;
+	char *personal, *language, *spelling = NULL, *jargon = NULL, *encoding = NULL;
+	int personal_len, language_len, spelling_len = 0, jargon_len = 0, encoding_len = 0;
 	long mode = 0L,  speed = 0L;
-	int argc;
+	int argc = ZEND_NUM_ARGS();
 	int ind;
 
 #ifdef PHP_WIN32
@@ -278,10 +373,10 @@ static PHP_FUNCTION(pspell_new_personal)
 	PspellCanHaveError *ret;
 	PspellManager *manager;
 	PspellConfig *config;
-	
-	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 6 || zend_get_parameters_ex(argc,&personal,&language,&spelling,&jargon,&encoding,&pmode) == FAILURE) {
-		WRONG_PARAM_COUNT;
+
+	if (zend_parse_parameters(argc TSRMLS_CC, "ps|sssl", &personal, &personal_len, &language, &language_len, 
+		&spelling, &spelling_len, &jargon, &jargon_len, &encoding, &encoding_len, &mode) == FAILURE) {
+		return;
 	}
 
 	config = new_pspell_config();
@@ -290,75 +385,59 @@ static PHP_FUNCTION(pspell_new_personal)
 	/* If aspell was installed using installer, we should have a key
 	 * pointing to the location of the dictionaries
 	 */
-	if(0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+	if (0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+		LONG result;
 		dwLen = sizeof(aspell_dir) - 1;
-		RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
+		result = RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
 		RegCloseKey(hkey);
-		strcpy(data_dir, aspell_dir);
-		strcat(data_dir, "\\data");
-		strcpy(dict_dir, aspell_dir);
-		strcat(dict_dir, "\\dict");
+		if (result == ERROR_SUCCESS) {
+			strlcpy(data_dir, aspell_dir, sizeof(data_dir));
+			strlcat(data_dir, "\\data", sizeof(data_dir));
+			strlcpy(dict_dir, aspell_dir, sizeof(dict_dir));
+			strlcat(dict_dir, "\\dict", sizeof(dict_dir));
 
-		pspell_config_replace(config, "data-dir", data_dir);
-		pspell_config_replace(config, "dict-dir", dict_dir);
+			pspell_config_replace(config, "data-dir", data_dir);
+			pspell_config_replace(config, "dict-dir", dict_dir);
+		}
 	}
 #endif
 
-	convert_to_string_ex(personal);
-
-	if (PG(safe_mode) && (!php_checkuid(Z_STRVAL_PP(personal), NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+	if (php_check_open_basedir(personal TSRMLS_CC)) {
 		delete_pspell_config(config);
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(Z_STRVAL_PP(personal) TSRMLS_CC)) {
-		delete_pspell_config(config);
-		RETURN_FALSE;
-	}
-
-	pspell_config_replace(config, "personal", Z_STRVAL_PP(personal));
+	pspell_config_replace(config, "personal", personal);
 	pspell_config_replace(config, "save-repl", "false");
 
-	convert_to_string_ex(language);
-	pspell_config_replace(config, "language-tag", Z_STRVAL_PP(language));
+	pspell_config_replace(config, "language-tag", language);
 
-	if(argc > 2){
-		convert_to_string_ex(spelling);
-	 	if(Z_STRLEN_PP(spelling) > 0){
-			pspell_config_replace(config, "spelling", Z_STRVAL_PP(spelling));
-		}
+	if (spelling_len) {
+		pspell_config_replace(config, "spelling", spelling);
 	}
 
-	if(argc > 3){
-		convert_to_string_ex(jargon);
-		if(Z_STRLEN_PP(jargon) > 0){
-			pspell_config_replace(config, "jargon", Z_STRVAL_PP(jargon));
-		}
+	if (jargon_len) {
+		pspell_config_replace(config, "jargon", jargon);
 	}
 
-	if(argc > 4){
-		convert_to_string_ex(encoding);
-		if(Z_STRLEN_PP(encoding) > 0){
-			pspell_config_replace(config, "encoding", Z_STRVAL_PP(encoding));
-		}
+	if (encoding_len) {
+		pspell_config_replace(config, "encoding", encoding);
 	}
 
-	if(argc > 5){
-		convert_to_long_ex(pmode);
-		mode = Z_LVAL_PP(pmode);
+	if (argc > 5) {
 		speed = mode & PSPELL_SPEED_MASK_INTERNAL;
 
 		/* First check what mode we want (how many suggestions) */
-		if(speed == PSPELL_FAST){
+		if (speed == PSPELL_FAST) {
 			pspell_config_replace(config, "sug-mode", "fast");
-		}else if(speed == PSPELL_NORMAL){
+		} else if (speed == PSPELL_NORMAL) {
 			pspell_config_replace(config, "sug-mode", "normal");
-		}else if(speed == PSPELL_BAD_SPELLERS){
+		} else if (speed == PSPELL_BAD_SPELLERS) {
 			pspell_config_replace(config, "sug-mode", "bad-spellers");
 		}
 		
 		/* Then we see if run-together words should be treated as valid components */
-		if(mode & PSPELL_RUN_TOGETHER){
+		if (mode & PSPELL_RUN_TOGETHER) {
 			pspell_config_replace(config, "run-together", "true");
 		}
 	}
@@ -366,14 +445,14 @@ static PHP_FUNCTION(pspell_new_personal)
 	ret = new_pspell_manager(config);
 	delete_pspell_config(config);
 
-	if(pspell_error_number(ret) != 0){
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+	if (pspell_error_number(ret) != 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s", pspell_error_message(ret));
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
 	manager = to_pspell_manager(ret);
-	ind = zend_list_insert(manager, le_pspell);
+	ind = zend_list_insert(manager, le_pspell TSRMLS_CC);
 	RETURN_LONG(ind);
 }
 /* }}} */
@@ -382,32 +461,28 @@ static PHP_FUNCTION(pspell_new_personal)
    Load a dictionary based on the given config */
 static PHP_FUNCTION(pspell_new_config)
 {
-	int type;
-	zval **conf;
-	int argc;
-	int ind;
-	
+	int type, ind;
+	long conf;	
 	PspellCanHaveError *ret;
 	PspellManager *manager;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 1 || zend_get_parameters_ex(argc,&conf) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &conf) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
 	ret = new_pspell_manager(config);
 
-	if(pspell_error_number(ret) != 0){
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+	if (pspell_error_number(ret) != 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s", pspell_error_message(ret));
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
 	manager = to_pspell_manager(ret);
-	ind = zend_list_insert(manager, le_pspell);
+	ind = zend_list_insert(manager, le_pspell TSRMLS_CC);
 	RETURN_LONG(ind);
 }
 /* }}} */
@@ -416,23 +491,20 @@ static PHP_FUNCTION(pspell_new_config)
    Returns true if word is valid */
 static PHP_FUNCTION(pspell_check)
 {
-	int type;
-	zval **scin,**word;
+	int type, word_len;
+	long scin;
+	char *word;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc, &scin,&word) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &scin, &word, &word_len) == FAILURE) {
+		return;
 	}
-    
-	convert_to_string_ex(word);
 
 	PSPELL_FETCH_MANAGER;
 
-	if(pspell_manager_check(manager, Z_STRVAL_PP(word))){
+	if (pspell_manager_check(manager, word)) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		RETURN_FALSE;
 	}
 }
@@ -442,32 +514,31 @@ static PHP_FUNCTION(pspell_check)
    Returns array of suggestions */
 static PHP_FUNCTION(pspell_suggest)
 {
-	zval **scin, **word;
-	int argc;
+	long scin;
+	char *word;
+	int word_len;
 	PspellManager *manager;
 	int type;
 	const PspellWordList *wl;
 	const char *sug;
 
-	argc = ZEND_NUM_ARGS();
-	if(argc != 2 || zend_get_parameters_ex(argc, &scin,&word) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &scin, &word, &word_len) == FAILURE) {
+		return;
 	}
-    
-	convert_to_string_ex(word);
+
 	PSPELL_FETCH_MANAGER;
 
 	array_init(return_value);
 
-	wl = pspell_manager_suggest(manager, Z_STRVAL_PP(word));
-	if(wl){
+	wl = pspell_manager_suggest(manager, word);
+	if (wl) {
 		PspellStringEmulation *els = pspell_word_list_elements(wl);
-		while((sug = pspell_string_emulation_next(els)) != 0){
+		while ((sug = pspell_string_emulation_next(els)) != 0) {
 			add_next_index_string(return_value,(char *)sug,1);
 		}
 		delete_pspell_string_emulation(els);
-	}else{
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL had a problem. details: %s ", pspell_manager_error_message(manager));
+	} else {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL had a problem. details: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
 }
@@ -477,24 +548,21 @@ static PHP_FUNCTION(pspell_suggest)
    Notify the dictionary of a user-selected replacement */
 static PHP_FUNCTION(pspell_store_replacement)
 {
-	int type;
-	zval **scin,**miss,**corr;
+	int type, miss_len, corr_len;
+	long scin;
+	char *miss, *corr;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 3 || zend_get_parameters_ex(argc, &scin,&miss,&corr) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lss", &scin, &miss, &miss_len, &corr, &corr_len) == FAILURE) {
+		return;
 	}
-    
-	convert_to_string_ex(miss);
-	convert_to_string_ex(corr);
+
 	PSPELL_FETCH_MANAGER;
 
-	pspell_manager_store_replacement(manager, Z_STRVAL_PP(miss), Z_STRVAL_PP(corr));
-	if(pspell_manager_error_number(manager) == 0){
+	pspell_manager_store_replacement(manager, miss, corr);
+	if (pspell_manager_error_number(manager) == 0) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "pspell_store_replacement() gave error: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
@@ -505,28 +573,26 @@ static PHP_FUNCTION(pspell_store_replacement)
    Adds a word to a personal list */
 static PHP_FUNCTION(pspell_add_to_personal)
 {
-	int type;
-	zval **scin,**word;
+	int type, word_len;
+	long scin;
+	char *word;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc, &scin,&word) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &scin, &word, &word_len) == FAILURE) {
+		return;
 	}
-    
-	convert_to_string_ex(word);
+
 	PSPELL_FETCH_MANAGER;
 
 	/*If the word is empty, we have to return; otherwise we'll segfault! ouch!*/
-	if(Z_STRLEN_PP(word) == 0){
+	if (word_len == 0) {
 		RETURN_FALSE;
 	}
 	
-	pspell_manager_add_to_personal(manager, Z_STRVAL_PP(word));
-	if(pspell_manager_error_number(manager) == 0){
+	pspell_manager_add_to_personal(manager, word);
+	if (pspell_manager_error_number(manager) == 0) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "pspell_add_to_personal() gave error: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
@@ -537,28 +603,26 @@ static PHP_FUNCTION(pspell_add_to_personal)
    Adds a word to the current session */
 static PHP_FUNCTION(pspell_add_to_session)
 {
-	int type;
-	zval **scin,**word;
+	int type, word_len;
+	long scin;
+	char *word;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc, &scin,&word) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &scin, &word, &word_len) == FAILURE) {
+		return;
 	}
-    
-	convert_to_string_ex(word);
+
 	PSPELL_FETCH_MANAGER;
 
 	/*If the word is empty, we have to return; otherwise we'll segfault! ouch!*/
-	if(Z_STRLEN_PP(word) == 0){
+	if (word_len == 0) {
 		RETURN_FALSE;
 	}
 
-	pspell_manager_add_to_session(manager, Z_STRVAL_PP(word));
-	if(pspell_manager_error_number(manager) == 0){
+	pspell_manager_add_to_session(manager, word);
+	if (pspell_manager_error_number(manager) == 0) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "pspell_add_to_session() gave error: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
@@ -570,21 +634,19 @@ static PHP_FUNCTION(pspell_add_to_session)
 static PHP_FUNCTION(pspell_clear_session)
 {
 	int type;
-	zval **scin;
+	long scin;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 1 || zend_get_parameters_ex(argc, &scin) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &scin) == FAILURE) {
+		return;
 	}
     
 	PSPELL_FETCH_MANAGER;	
 
 	pspell_manager_clear_session(manager);
-	if(pspell_manager_error_number(manager) == 0){
+	if (pspell_manager_error_number(manager) == 0) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "pspell_clear_session() gave error: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
@@ -596,22 +658,20 @@ static PHP_FUNCTION(pspell_clear_session)
 static PHP_FUNCTION(pspell_save_wordlist)
 {
 	int type;
-	zval **scin;
+	long scin;
 	PspellManager *manager;
 
-	int argc;
-	argc = ZEND_NUM_ARGS();
-	if (argc != 1 || zend_get_parameters_ex(argc, &scin) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &scin) == FAILURE) {
+		return;
 	}
     
 	PSPELL_FETCH_MANAGER;	
 
 	pspell_manager_save_all_word_lists(manager);
 
-	if(pspell_manager_error_number(manager) == 0){
+	if (pspell_manager_error_number(manager) == 0) {
 		RETURN_TRUE;
-	}else{
+	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "pspell_save_wordlist() gave error: %s", pspell_manager_error_message(manager));
 		RETURN_FALSE;
 	}
@@ -623,10 +683,9 @@ static PHP_FUNCTION(pspell_save_wordlist)
    Create a new config to be used later to create a manager */
 static PHP_FUNCTION(pspell_config_create)
 {
-	zval **language,**spelling,**jargon,**encoding;
-	int argc;
+	char *language, *spelling = NULL, *jargon = NULL, *encoding = NULL;
+	int language_len, spelling_len = 0, jargon_len = 0, encoding_len = 0;
 	int ind;
-
 	PspellConfig *config;
 
 #ifdef PHP_WIN32
@@ -637,9 +696,9 @@ static PHP_FUNCTION(pspell_config_create)
 	DWORD dwType,dwLen;
 #endif
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc < 1 || argc > 4 || zend_get_parameters_ex(argc,&language,&spelling,&jargon,&encoding) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|sss", &language, &language_len, &spelling, &spelling_len, 
+		&jargon, &jargon_len, &encoding, &encoding_len) == FAILURE) {
+		return;
 	}
 
 	config = new_pspell_config();
@@ -648,49 +707,42 @@ static PHP_FUNCTION(pspell_config_create)
     /* If aspell was installed using installer, we should have a key
      * pointing to the location of the dictionaries
      */
-	if(0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+	if (0 == RegOpenKey(HKEY_LOCAL_MACHINE, "Software\\Aspell", &hkey)) {
+		LONG result;
 		dwLen = sizeof(aspell_dir) - 1;
-		RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
+		result = RegQueryValueEx(hkey, "", NULL, &dwType, (LPBYTE)&aspell_dir, &dwLen);
 		RegCloseKey(hkey);
-		strcpy(data_dir, aspell_dir);
-		strcat(data_dir, "\\data");
-		strcpy(dict_dir, aspell_dir);
-		strcat(dict_dir, "\\dict");
+		if (result == ERROR_SUCCESS) {
+			strlcpy(data_dir, aspell_dir, sizeof(data_dir));
+			strlcat(data_dir, "\\data", sizeof(data_dir));
+			strlcpy(dict_dir, aspell_dir, sizeof(dict_dir));
+			strlcat(dict_dir, "\\dict", sizeof(dict_dir));
 
-		pspell_config_replace(config, "data-dir", data_dir);
-		pspell_config_replace(config, "dict-dir", dict_dir);
+			pspell_config_replace(config, "data-dir", data_dir);
+			pspell_config_replace(config, "dict-dir", dict_dir);
+		}
 	}
 #endif
 
-	convert_to_string_ex(language);
-	pspell_config_replace(config, "language-tag", Z_STRVAL_PP(language));
+	pspell_config_replace(config, "language-tag", language);
 
-	if(argc > 1){
-		convert_to_string_ex(spelling);
-	 	if(Z_STRLEN_PP(spelling) > 0){
-			pspell_config_replace(config, "spelling", Z_STRVAL_PP(spelling));
-		}
+ 	if (spelling_len) {
+		pspell_config_replace(config, "spelling", spelling);
 	}
 
-	if(argc > 2){
-		convert_to_string_ex(jargon);
-		if(Z_STRLEN_PP(jargon) > 0){
-			pspell_config_replace(config, "jargon", Z_STRVAL_PP(jargon));
-		}
+	if (jargon_len) {
+		pspell_config_replace(config, "jargon", jargon);
 	}
 
-	if(argc > 3){
-		convert_to_string_ex(encoding);
-		if(Z_STRLEN_PP(encoding) > 0){
-			pspell_config_replace(config, "encoding", Z_STRVAL_PP(encoding));
-		}
+	if (encoding_len) {
+		pspell_config_replace(config, "encoding", encoding);
 	}
 
 	/* By default I do not want to write anything anywhere because it'll try to write to $HOME
 	which is not what we want */
 	pspell_config_replace(config, "save-repl", "false");
 
-	ind = zend_list_insert(config, le_pspell_config);
+	ind = zend_list_insert(config, le_pspell_config TSRMLS_CC);
 	RETURN_LONG(ind);
 }
 /* }}} */
@@ -700,20 +752,17 @@ static PHP_FUNCTION(pspell_config_create)
 static PHP_FUNCTION(pspell_config_runtogether)
 {
 	int type;
-	zval **conf, **runtogether;
-	int argc;
-
+	long conf;
+	zend_bool runtogether;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc,&conf,&runtogether) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lb", &conf, &runtogether) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;	
 
-	convert_to_boolean_ex(runtogether);
-	pspell_config_replace(config, "run-together", Z_LVAL_PP(runtogether) ? "true" : "false");
+	pspell_config_replace(config, "run-together", runtogether ? "true" : "false");
 	
 	RETURN_TRUE;
 }
@@ -724,26 +773,21 @@ static PHP_FUNCTION(pspell_config_runtogether)
 static PHP_FUNCTION(pspell_config_mode)
 {
 	int type;
-	zval **conf, **mode;
-	int argc;
-
+	long conf, mode;
 	PspellConfig *config;
-	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc,&conf,&mode) == FAILURE) {
-		WRONG_PARAM_COUNT;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &conf, &mode) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
-	convert_to_long_ex(mode);
-
 	/* First check what mode we want (how many suggestions) */
-	if(Z_LVAL_PP(mode) == PSPELL_FAST){
+	if (mode == PSPELL_FAST) {
 		pspell_config_replace(config, "sug-mode", "fast");
-	}else if(Z_LVAL_PP(mode) == PSPELL_NORMAL){
+	} else if (mode == PSPELL_NORMAL) {
 		pspell_config_replace(config, "sug-mode", "normal");
-	}else if(Z_LVAL_PP(mode) == PSPELL_BAD_SPELLERS){
+	} else if (mode == PSPELL_BAD_SPELLERS) {
 		pspell_config_replace(config, "sug-mode", "bad-spellers");
 	}
 
@@ -756,25 +800,17 @@ static PHP_FUNCTION(pspell_config_mode)
 static PHP_FUNCTION(pspell_config_ignore)
 {
 	int type;
-	zval **conf, **pignore;
-	int argc;
-
 	char ignore_str[MAX_LENGTH_OF_LONG + 1];	
-	long ignore = 0L;
-
+	long conf, ignore = 0L;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc,&conf,&pignore) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &conf, &ignore) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
-	convert_to_long_ex(pignore);
-	ignore = Z_LVAL_PP(pignore);
-
-	sprintf(ignore_str, "%ld", ignore);
+	snprintf(ignore_str, sizeof(ignore_str), "%ld", ignore);
 
 	pspell_config_replace(config, "ignore", ignore_str);
 	RETURN_TRUE;
@@ -784,28 +820,22 @@ static PHP_FUNCTION(pspell_config_ignore)
 static void pspell_config_path(INTERNAL_FUNCTION_PARAMETERS, char *option)
 {
 	int type;
-	zval **conf, **value;
-	int argc;
+	long conf;
+	char *value;
+	int value_len;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc, &conf, &value) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lp", &conf, &value, &value_len) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
-	convert_to_string_ex(value);
-
-	if (PG(safe_mode) && (!php_checkuid(Z_STRVAL_PP(value), NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+	if (php_check_open_basedir(value TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(Z_STRVAL_PP(value) TSRMLS_CC)) {
-		RETURN_FALSE;
-	}
-
-	pspell_config_replace(config, option, Z_STRVAL_PP(value));
+	pspell_config_replace(config, option, value);
 
 	RETURN_TRUE;
 }
@@ -839,31 +869,24 @@ static PHP_FUNCTION(pspell_config_data_dir)
 static PHP_FUNCTION(pspell_config_repl)
 {
 	int type;
-	zval **conf, **repl;
-	int argc;
-
+	long conf;
+	char *repl;
+	int repl_len;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc,&conf,&repl) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lp", &conf, &repl, &repl_len) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
 	pspell_config_replace(config, "save-repl", "true");
 
-	convert_to_string_ex(repl);
-
-	if (PG(safe_mode) && (!php_checkuid(Z_STRVAL_PP(repl), NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+	if (php_check_open_basedir(repl TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(Z_STRVAL_PP(repl) TSRMLS_CC)) {
-		RETURN_FALSE;
-	}
-
-	pspell_config_replace(config, "repl", Z_STRVAL_PP(repl));
+	pspell_config_replace(config, "repl", repl);
 
 	RETURN_TRUE;
 }
@@ -874,20 +897,17 @@ static PHP_FUNCTION(pspell_config_repl)
 static PHP_FUNCTION(pspell_config_save_repl)
 {
 	int type;
-	zval **conf, **save;
-	int argc;
-
+	long conf;
+	zend_bool save;
 	PspellConfig *config;
 	
-	argc = ZEND_NUM_ARGS();
-	if (argc != 2 || zend_get_parameters_ex(argc,&conf,&save) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lb", &conf, &save) == FAILURE) {
+		return;
 	}
 
 	PSPELL_FETCH_CONFIG;
 
-	convert_to_boolean_ex(save);
-	pspell_config_replace(config, "save-repl", Z_LVAL_PP(save) ? "true" : "false");
+	pspell_config_replace(config, "save-repl", save ? "true" : "false");
 
 	RETURN_TRUE;
 }
