@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
+/* $Id: formatted_print.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include <math.h>				/* modf() */
 #include "php.h"
@@ -37,10 +37,7 @@
 #define ADJ_WIDTH 1
 #define ADJ_PRECISION 2
 #define NUM_BUF_SIZE 500
-#define	NDIG 80
-#define FLOAT_DIGITS 6
 #define FLOAT_PRECISION 6
-#define MAX_FLOAT_DIGITS 38
 #define MAX_FLOAT_PRECISION 53
 
 #if 0
