@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,7 +15,7 @@
    | Author: Wez Furlong <wez@thebrainroom.com>                           |
    +----------------------------------------------------------------------+
  */
-/* $Id: proc_open.c,v 1.36.2.1.2.17.2.8 2009/06/09 00:25:37 pajoye Exp $ */
+/* $Id: proc_open.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #if 0 && (defined(__linux__) || defined(sun) || defined(__IRIX__))
 # define _BSD_SOURCE 		/* linux wants this when XOPEN mode is on */
@@ -28,8 +28,8 @@
 #include <stdio.h>
 #include <ctype.h>
 #include "php_string.h"
-#include "safe_mode.h"
 #include "ext/standard/head.h"
+#include "ext/standard/basic_functions.h"
 #include "ext/standard/file.h"
 #include "exec.h"
 #include "php_globals.h"
@@ -87,13 +87,13 @@ static php_process_env_t _php_array_to_envp(zval *environment, int is_persistent
 	HashPosition pos;
 
 	memset(&env, 0, sizeof(env));
-	
+
 	if (!environment) {
 		return env;
 	}
-	
+
 	cnt = zend_hash_num_elements(Z_ARRVAL_P(environment));
-	
+
 	if (cnt < 1) {
 #ifndef PHP_WIN32
 		env.envarray = (char **) pecalloc(1, sizeof(char *), is_persistent);
@@ -111,15 +111,15 @@ static php_process_env_t _php_array_to_envp(zval *environment, int is_persistent
 	for (zend_hash_internal_pointer_reset_ex(target_hash, &pos);
 			zend_hash_get_current_data_ex(target_hash, (void **) &element, &pos) == SUCCESS;
 			zend_hash_move_forward_ex(target_hash, &pos)) {
-		
+
 		convert_to_string_ex(element);
 		el_len = Z_STRLEN_PP(element);
 		if (el_len == 0) {
 			continue;
 		}
-		
+
 		sizeenv += el_len+1;
-		
+
 		switch (zend_hash_get_current_key_ex(target_hash, &string_key, &string_length, &num_key, 0, &pos)) {
 			case HASH_KEY_IS_STRING:
 				if (string_length == 0) {
@@ -138,25 +138,26 @@ static php_process_env_t _php_array_to_envp(zval *environment, int is_persistent
 	for (zend_hash_internal_pointer_reset_ex(target_hash, &pos);
 			zend_hash_get_current_data_ex(target_hash, (void **) &element, &pos) == SUCCESS;
 			zend_hash_move_forward_ex(target_hash, &pos)) {
-		
+
 		convert_to_string_ex(element);
 		el_len = Z_STRLEN_PP(element);
-		
+
 		if (el_len == 0) {
 			continue;
 		}
-		
+
 		data = Z_STRVAL_PP(element);
 		switch (zend_hash_get_current_key_ex(target_hash, &string_key, &string_length, &num_key, 0, &pos)) {
 			case HASH_KEY_IS_STRING:
 				if (string_length == 0) {
 					continue;
 				}
+
 				l = string_length + el_len + 1;
 				memcpy(p, string_key, string_length);
-				strcat(p, "=");
-				strcat(p, data);
-				
+				strncat(p, "=", 1);
+				strncat(p, data, el_len);
+
 #ifndef PHP_WIN32
 				*ep = p;
 				++ep;
@@ -174,10 +175,10 @@ static php_process_env_t _php_array_to_envp(zval *environment, int is_persistent
 			case HASH_KEY_NON_EXISTANT:
 				break;
 		}
-	}	
+	}
+
+	assert((uint)(p - env.envp) <= sizeenv);
 
-	assert(p - env.envp <= sizeenv);
-	
 	zend_hash_internal_pointer_reset_ex(target_hash, &pos);
 
 	return env;
@@ -217,82 +218,34 @@ static void proc_open_rsrc_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 			proc->pipes[i] = 0;
 		}
 	}
-	
+
 #ifdef PHP_WIN32
-	
 	WaitForSingleObject(proc->childHandle, INFINITE);
 	GetExitCodeProcess(proc->childHandle, &wstatus);
 	FG(pclose_ret) = wstatus;
 	CloseHandle(proc->childHandle);
-	
+
 #elif HAVE_SYS_WAIT_H
-	
+
 	do {
 		wait_pid = waitpid(proc->child, &wstatus, 0);
 	} while (wait_pid == -1 && errno == EINTR);
-	
-	if (wait_pid == -1)
+
+	if (wait_pid == -1) {
 		FG(pclose_ret) = -1;
-	else {
+	} else {
 		if (WIFEXITED(wstatus))
 			wstatus = WEXITSTATUS(wstatus);
 		FG(pclose_ret) = wstatus;
 	}
-	
+
 #else
 	FG(pclose_ret) = -1;
 #endif
 	_php_free_envp(proc->env, proc->is_persistent);
 	pefree(proc->command, proc->is_persistent);
 	pefree(proc, proc->is_persistent);
-	
-}
-/* }}} */
 
-/* {{{ php_make_safe_mode_command */
-static int php_make_safe_mode_command(char *cmd, char **safecmd, int is_persistent TSRMLS_DC)
-{
-	int lcmd, larg0;
-	char *space, *sep, *arg0;
-
-	if (!PG(safe_mode)) {
-		*safecmd = pestrdup(cmd, is_persistent);
-		return SUCCESS;
-	}
-
-	lcmd = strlen(cmd);
-
-	arg0 = estrndup(cmd, lcmd);
-
-	space = memchr(arg0, ' ', lcmd);
-	if (space) {
-		*space = '\0';
-		larg0 = space - arg0;
-	} else {
-		larg0 = lcmd;
-	}
-
-	if (php_memnstr(arg0, "..", sizeof("..")-1, arg0 + larg0)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "No '..' components allowed in path");
-		efree(arg0);
-		return FAILURE;
-	}
-
-	sep = zend_memrchr(arg0, PHP_DIR_SEPARATOR, larg0);
-
-	spprintf(safecmd, 0, "%s%s%s%s", PG(safe_mode_exec_dir), (sep ? sep : "/"), (sep ? "" : arg0), (space ? cmd + larg0 : ""));
-
-	efree(arg0);
-	arg0 = php_escape_shell_cmd(*safecmd);
-	efree(*safecmd);
-	if (is_persistent) {
-		*safecmd = pestrdup(arg0, 1);
-		efree(arg0);
-	} else {
-		*safecmd = arg0;
-	}
-
-	return SUCCESS;
 }
 /* }}} */
 
@@ -311,13 +264,13 @@ PHP_FUNCTION(proc_terminate)
 	zval *zproc;
 	struct php_process_handle *proc;
 	long sig_no = SIGTERM;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|l", &zproc, &sig_no) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	ZEND_FETCH_RESOURCE(proc, struct php_process_handle *, &zproc, -1, "process", le_proc_open);
-	
+
 #ifdef PHP_WIN32
 	if (TerminateProcess(proc->childHandle, 255)) {
 		RETURN_TRUE;
@@ -340,13 +293,13 @@ PHP_FUNCTION(proc_close)
 {
 	zval *zproc;
 	struct php_process_handle *proc;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zproc) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	ZEND_FETCH_RESOURCE(proc, struct php_process_handle *, &zproc, -1, "process", le_proc_open);
-	
+
 	zend_list_delete(Z_LVAL_P(zproc));
 	RETURN_LONG(FG(pclose_ret));
 }
@@ -366,7 +319,7 @@ PHP_FUNCTION(proc_get_status)
 #endif
 	int running = 1, signaled = 0, stopped = 0;
 	int exitcode = -1, termsig = 0, stopsig = 0;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zproc) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -377,19 +330,19 @@ PHP_FUNCTION(proc_get_status)
 
 	add_assoc_string(return_value, "command", proc->command, 1);
 	add_assoc_long(return_value, "pid", (long) proc->child);
-	
+
 #ifdef PHP_WIN32
-	
+
 	GetExitCodeProcess(proc->childHandle, &wstatus);
 
 	running = wstatus == STILL_ACTIVE;
-	exitcode == STILL_ACTIVE ? -1 : wstatus;
-	
+	exitcode = running ? -1 : wstatus;
+
 #elif HAVE_SYS_WAIT_H
-	
+
 	errno = 0;
 	wait_pid = waitpid(proc->child, &wstatus, WNOHANG|WUNTRACED);
-	
+
 	if (wait_pid == proc->child) {
 		if (WIFEXITED(wstatus)) {
 			running = 0;
@@ -424,15 +377,14 @@ PHP_FUNCTION(proc_get_status)
 
 /* {{{ handy definitions for portability/readability */
 #ifdef PHP_WIN32
-# define pipe(pair)		(CreatePipe(&pair[0], &pair[1], &security, 2048L) ? 0 : -1)
+# define pipe(pair)		(CreatePipe(&pair[0], &pair[1], &security, 0) ? 0 : -1)
 
 # define COMSPEC_NT	"cmd.exe"
-# define COMSPEC_9X	"command.com"
 
 static inline HANDLE dup_handle(HANDLE src, BOOL inherit, BOOL closeorig)
 {
 	HANDLE copy, self = GetCurrentProcess();
-	
+
 	if (!DuplicateHandle(self, src, self, &copy, 0, inherit, DUPLICATE_SAME_ACCESS |
 				(closeorig ? DUPLICATE_CLOSE_SOURCE : 0)))
 		return NULL;
@@ -512,9 +464,7 @@ PHP_FUNCTION(proc_open)
 		RETURN_FALSE;
 	}
 
-	if (FAILURE == php_make_safe_mode_command(command, &command, is_persistent TSRMLS_CC)) {
-		RETURN_FALSE;
-	}
+	command = pestrdup(command, is_persistent);
 
 #ifdef PHP_WIN32
 	if (other_options) {
@@ -524,16 +474,16 @@ PHP_FUNCTION(proc_open)
 			    Z_LVAL_PP(item)) {
 				suppress_errors = 1;
 			}
-		}	
+		}
 		if (SUCCESS == zend_hash_find(Z_ARRVAL_P(other_options), "bypass_shell", sizeof("bypass_shell"), (void**)&item)) {
 			if ((Z_TYPE_PP(item) == IS_BOOL || Z_TYPE_PP(item) == IS_LONG) &&
 			    Z_LVAL_PP(item)) {
 				bypass_shell = 1;
 			}
-		}	
+		}
 	}
 #endif
-	
+
 	command_len = strlen(command);
 
 	if (environment) {
@@ -551,7 +501,7 @@ PHP_FUNCTION(proc_open)
 	security.bInheritHandle = TRUE;
 	security.lpSecurityDescriptor = NULL;
 #endif
-	
+
 	/* walk the descriptor spec and set up files/pipes */
 	zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(descriptorspec), &pos);
 	while (zend_hash_get_current_data_ex(Z_ARRVAL_P(descriptorspec), (void **)&descitem, &pos) == SUCCESS) {
@@ -666,7 +616,7 @@ PHP_FUNCTION(proc_open)
 
 				/* try a wrapper */
 				stream = php_stream_open_wrapper(Z_STRVAL_PP(zfile), Z_STRVAL_PP(zmode),
-						ENFORCE_SAFE_MODE|REPORT_ERRORS|STREAM_WILL_CAST, NULL);
+						REPORT_ERRORS|STREAM_WILL_CAST, NULL);
 
 				/* force into an fd */
 				if (stream == NULL || FAILURE == php_stream_cast(stream,
@@ -725,14 +675,24 @@ PHP_FUNCTION(proc_open)
 	}
 
 #ifdef PHP_WIN32
+	if (cwd == NULL) {
+		char cur_cwd[MAXPATHLEN];
+		char *getcwd_result;
+		getcwd_result = VCWD_GETCWD(cur_cwd, MAXPATHLEN);
+		if (!getcwd_result) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot get current directory");
+			goto exit_fail;
+		}
+	}
+
 	memset(&si, 0, sizeof(si));
 	si.cb = sizeof(si);
 	si.dwFlags = STARTF_USESTDHANDLES;
-	
+
 	si.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
 	si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
 	si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
-	
+
 	/* redirect stdin/stdout/stderr if requested */
 	for (i = 0; i < ndesc; i++) {
 		switch(descriptors[i].index) {
@@ -748,13 +708,13 @@ PHP_FUNCTION(proc_open)
 		}
 	}
 
-	
+
 	memset(&pi, 0, sizeof(pi));
-	
+
 	if (suppress_errors) {
 		old_error_mode = SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOGPFAULTERRORBOX);
 	}
-	
+
 	dwCreateFlags = NORMAL_PRIORITY_CLASS;
 	if(strcmp(sapi_module.name, "cli") != 0) {
 		dwCreateFlags |= CREATE_NO_WINDOW;
@@ -763,17 +723,17 @@ PHP_FUNCTION(proc_open)
 	if (bypass_shell) {
 		newprocok = CreateProcess(NULL, command, &security, &security, TRUE, dwCreateFlags, env.envp, cwd, &si, &pi);
 	} else {
-		spprintf(&command_with_cmd, 0, "%s /c %s", GetVersion() < 0x80000000 ? COMSPEC_NT : COMSPEC_9X, command);
+		spprintf(&command_with_cmd, 0, "%s /c %s", COMSPEC_NT, command);
 
 		newprocok = CreateProcess(NULL, command_with_cmd, &security, &security, TRUE, dwCreateFlags, env.envp, cwd, &si, &pi);
-		
+
 		efree(command_with_cmd);
 	}
 
 	if (suppress_errors) {
 		SetErrorMode(old_error_mode);
 	}
-	
+
 	if (FALSE == newprocok) {
 		DWORD dw = GetLastError();
 
@@ -858,7 +818,7 @@ PHP_FUNCTION(proc_open)
 			tcsetpgrp(0, my_pid);
 		}
 #endif
-		
+
 		/* close those descriptors that we just opened for the parent stuff,
 		 * dup new descriptors into required descriptors and close the original
 		 * cruft */
@@ -880,11 +840,11 @@ PHP_FUNCTION(proc_open)
 			close(slave_pty);
 		}
 #endif
-		
+
 		if (cwd) {
-			chdir(cwd);
+			php_ignore_value(chdir(cwd));
 		}
-		
+
 		if (env.envarray) {
 			execle("/bin/sh", "sh", "-c", command, NULL, env.envarray);
 		} else {
@@ -978,7 +938,7 @@ PHP_FUNCTION(proc_open)
 
 					/* nasty hack; don't copy it */
 					stream->flags |= PHP_STREAM_FLAG_NO_SEEK;
-					
+
 					MAKE_STD_ZVAL(retfp);
 					php_stream_to_zval(stream, retfp);
 					add_index_zval(pipes, descriptors[i].index, retfp);
